@@ -30,6 +30,9 @@
 
 #include "quick_hull.h"
 
+#include "core/templates/hash_map.h"
+#include "core/templates/hash_set.h"
+
 uint32_t QuickHull::debug_stop_after = 0xFFFFFFFF;
 
 Error QuickHull::build(const Vector<Vector3> &p_points, Geometry3D::MeshData &r_mesh) {
@@ -320,7 +323,7 @@ Error QuickHull::build(const Vector<Vector3> &p_points, Geometry3D::MeshData &r_
 
 		for (List<Face>::Element *&E : new_faces) {
 			Face &f2 = E->get();
-			if (f2.points_over.size() == 0) {
+			if (f2.points_over.is_empty()) {
 				faces.move_to_front(E);
 			}
 		}
