@@ -28,10 +28,9 @@
 /* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
 /**************************************************************************/
 
-#ifndef AUDIO_STREAM_INTERACTIVE_EDITOR_PLUGIN_H
-#define AUDIO_STREAM_INTERACTIVE_EDITOR_PLUGIN_H
+#pragma once
 
-#include "editor/editor_inspector.h"
+#include "editor/inspector/editor_inspector.h"
 #include "editor/plugins/editor_plugin.h"
 #include "scene/gui/dialogs.h"
 
@@ -106,5 +105,3 @@ public:
 
 	AudioStreamInteractiveEditorPlugin();
 };
-
-#endif // AUDIO_STREAM_INTERACTIVE_EDITOR_PLUGIN_H
