@@ -28,12 +28,11 @@
 /* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
 /**************************************************************************/
 
-#ifndef RENDERER_CANVAS_RENDER_RD_H
-#define RENDERER_CANVAS_RENDER_RD_H
+#pragma once
 
+#include "core/templates/lru.h"
 #include "servers/rendering/renderer_canvas_render.h"
-#include "servers/rendering/renderer_compositor.h"
-#include "servers/rendering/renderer_rd/pipeline_cache_rd.h"
+#include "servers/rendering/renderer_rd/pipeline_hash_map_rd.h"
 #include "servers/rendering/renderer_rd/shaders/canvas.glsl.gen.h"
 #include "servers/rendering/renderer_rd/shaders/canvas_occlusion.glsl.gen.h"
 #include "servers/rendering/renderer_rd/storage_rd/material_storage.h"
@@ -45,10 +44,12 @@ class RendererCanvasRenderRD : public RendererCanvasRender {
 		BASE_UNIFORM_SET = 0,
 		MATERIAL_UNIFORM_SET = 1,
 		TRANSFORMS_UNIFORM_SET = 2,
-		CANVAS_TEXTURE_UNIFORM_SET = 3,
+		BATCH_UNIFORM_SET = 3,
 	};
 
 	const int SAMPLERS_BINDING_FIRST_INDEX = 10;
+	// The size of the ring buffer to store GPU buffers. Triple-buffering the max expected frames in flight.
+	static const uint32_t BATCH_DATA_BUFFER_COUNT = 3;
 
 	enum ShaderVariant {
 		SHADER_VARIANT_QUAD,
@@ -57,41 +58,35 @@ class RendererCanvasRenderRD : public RendererCanvasRender {
 		SHADER_VARIANT_PRIMITIVE_POINTS,
 		SHADER_VARIANT_ATTRIBUTES,
 		SHADER_VARIANT_ATTRIBUTES_POINTS,
-		SHADER_VARIANT_QUAD_LIGHT,
-		SHADER_VARIANT_NINEPATCH_LIGHT,
-		SHADER_VARIANT_PRIMITIVE_LIGHT,
-		SHADER_VARIANT_PRIMITIVE_POINTS_LIGHT,
-		SHADER_VARIANT_ATTRIBUTES_LIGHT,
-		SHADER_VARIANT_ATTRIBUTES_POINTS_LIGHT,
 		SHADER_VARIANT_MAX
 	};
 
 	enum {
+		INSTANCE_FLAGS_LIGHT_COUNT_SHIFT = 0, // 4 bits for light count.
 
-		FLAGS_INSTANCING_MASK = 0x7F,
-		FLAGS_INSTANCING_HAS_COLORS = (1 << 7),
-		FLAGS_INSTANCING_HAS_CUSTOM_DATA = (1 << 8),
+		INSTANCE_FLAGS_CLIP_RECT_UV = (1 << 4),
+		INSTANCE_FLAGS_TRANSPOSE_RECT = (1 << 5),
+		INSTANCE_FLAGS_USE_MSDF = (1 << 6),
+		INSTANCE_FLAGS_USE_LCD = (1 << 7),
 
-		FLAGS_CLIP_RECT_UV = (1 << 9),
-		FLAGS_TRANSPOSE_RECT = (1 << 10),
+		INSTANCE_FLAGS_NINEPACH_DRAW_CENTER = (1 << 8),
+		INSTANCE_FLAGS_NINEPATCH_H_MODE_SHIFT = 9,
+		INSTANCE_FLAGS_NINEPATCH_V_MODE_SHIFT = 11,
 
-		FLAGS_CONVERT_ATTRIBUTES_TO_LINEAR = (1 << 11),
+		INSTANCE_FLAGS_SHADOW_MASKED_SHIFT = 13, // 16 bits.
+	};
 
-		FLAGS_NINEPACH_DRAW_CENTER = (1 << 12),
+	enum {
+		BATCH_FLAGS_INSTANCING_MASK = 0x7F,
+		BATCH_FLAGS_INSTANCING_HAS_COLORS = (1 << 7),
+		BATCH_FLAGS_INSTANCING_HAS_CUSTOM_DATA = (1 << 8),
 
-		FLAGS_USE_SKELETON = (1 << 15),
-		FLAGS_NINEPATCH_H_MODE_SHIFT = 16,
-		FLAGS_NINEPATCH_V_MODE_SHIFT = 18,
-		FLAGS_LIGHT_COUNT_SHIFT = 20,
+		BATCH_FLAGS_DEFAULT_NORMAL_MAP_USED = (1 << 9),
+		BATCH_FLAGS_DEFAULT_SPECULAR_MAP_USED = (1 << 10),
+	};
 
-		FLAGS_DEFAULT_NORMAL_MAP_USED = (1 << 26),
-		FLAGS_DEFAULT_SPECULAR_MAP_USED = (1 << 27),
-
-		FLAGS_USE_MSDF = (1 << 28),
-		FLAGS_USE_LCD = (1 << 29),
-
-		FLAGS_FLIP_H = (1 << 30),
-		FLAGS_FLIP_V = (1 << 31),
+	enum {
+		CANVAS_FLAGS_CONVERT_ATTRIBUTES_TO_LINEAR = (1 << 0),
 	};
 
 	enum {
@@ -111,83 +106,90 @@ class RendererCanvasRenderRD : public RendererCanvasRender {
 		MAX_RENDER_ITEMS = 256 * 1024,
 		MAX_LIGHT_TEXTURES = 1024,
 		MAX_LIGHTS_PER_ITEM = 16,
-		DEFAULT_MAX_LIGHTS_PER_RENDER = 256
+		MAX_LIGHTS_PER_RENDER = 256,
 	};
 
 	/****************/
 	/**** SHADER ****/
 	/****************/
 
-	enum PipelineVariant {
-		PIPELINE_VARIANT_QUAD,
-		PIPELINE_VARIANT_NINEPATCH,
-		PIPELINE_VARIANT_PRIMITIVE_TRIANGLES,
-		PIPELINE_VARIANT_PRIMITIVE_LINES,
-		PIPELINE_VARIANT_PRIMITIVE_POINTS,
-		PIPELINE_VARIANT_ATTRIBUTE_TRIANGLES,
-		PIPELINE_VARIANT_ATTRIBUTE_TRIANGLE_STRIP,
-		PIPELINE_VARIANT_ATTRIBUTE_LINES,
-		PIPELINE_VARIANT_ATTRIBUTE_LINES_STRIP,
-		PIPELINE_VARIANT_ATTRIBUTE_POINTS,
-		PIPELINE_VARIANT_QUAD_LCD_BLEND,
-		PIPELINE_VARIANT_MAX
-	};
-	enum PipelineLightMode {
-		PIPELINE_LIGHT_MODE_DISABLED,
-		PIPELINE_LIGHT_MODE_ENABLED,
-		PIPELINE_LIGHT_MODE_MAX
+	struct ShaderSpecialization {
+		union {
+			uint32_t packed_0;
+
+			struct {
+				uint32_t use_lighting : 1;
+			};
+		};
 	};
 
-	struct PipelineVariants {
-		PipelineCacheRD variants[PIPELINE_LIGHT_MODE_MAX][PIPELINE_VARIANT_MAX];
+	struct PipelineKey {
+		ShaderVariant variant = SHADER_VARIANT_MAX;
+		RD::FramebufferFormatID framebuffer_format_id = RD::INVALID_FORMAT_ID;
+		RD::VertexFormatID vertex_format_id = RD::INVALID_ID;
+		RD::RenderPrimitive render_primitive = RD::RENDER_PRIMITIVE_MAX;
+		ShaderSpecialization shader_specialization = {};
+		uint32_t lcd_blend = 0;
+		uint32_t ubershader = 0;
+
+		uint32_t hash() const {
+			uint32_t h = hash_murmur3_one_32(variant);
+			h = hash_murmur3_one_32(framebuffer_format_id, h);
+			h = hash_murmur3_one_64((uint64_t)vertex_format_id, h);
+			h = hash_murmur3_one_32(render_primitive, h);
+			h = hash_murmur3_one_32(shader_specialization.packed_0, h);
+			h = hash_murmur3_one_32(lcd_blend, h);
+			h = hash_murmur3_one_32(ubershader, h);
+			return hash_fmix32(h);
+		}
 	};
 
-	struct {
-		CanvasShaderRD canvas_shader;
-		RID default_version;
-		RID default_version_rd_shader;
-		RID quad_index_buffer;
-		RID quad_index_array;
-		PipelineVariants pipeline_variants;
-
-		ShaderCompiler compiler;
-	} shader;
-
 	struct CanvasShaderData : public RendererRD::MaterialStorage::ShaderData {
-		enum BlendMode { //used internally
-			BLEND_MODE_MIX,
-			BLEND_MODE_ADD,
-			BLEND_MODE_SUB,
-			BLEND_MODE_MUL,
-			BLEND_MODE_PMALPHA,
-			BLEND_MODE_DISABLED,
-		};
-
-		bool valid = false;
-		RID version;
-		PipelineVariants pipeline_variants;
-
 		Vector<ShaderCompiler::GeneratedCode::Texture> texture_uniforms;
+		int blend_mode = 0;
 
 		Vector<uint32_t> ubo_offsets;
 		uint32_t ubo_size = 0;
 
 		String code;
+		RID version;
+		PipelineHashMapRD<PipelineKey, CanvasShaderData, void (CanvasShaderData::*)(PipelineKey)> pipeline_hash_map;
+
+		static const uint32_t VERTEX_INPUT_MASKS_SIZE = SHADER_VARIANT_MAX * 2;
+		std::atomic<uint64_t> vertex_input_masks[VERTEX_INPUT_MASKS_SIZE] = {};
 
 		bool uses_screen_texture = false;
 		bool uses_screen_texture_mipmaps = false;
 		bool uses_sdf = false;
 		bool uses_time = false;
 
+		void _clear_vertex_input_mask_cache();
+		void _create_pipeline(PipelineKey p_pipeline_key);
 		virtual void set_code(const String &p_Code);
 		virtual bool is_animated() const;
 		virtual bool casts_shadows() const;
 		virtual RS::ShaderNativeSourceCode get_native_source_code() const;
+		virtual Pair<ShaderRD *, RID> get_native_shader_and_version() const;
+		RID get_shader(ShaderVariant p_shader_variant, bool p_ubershader) const;
+		uint64_t get_vertex_input_mask(ShaderVariant p_shader_variant, bool p_ubershader);
+		bool is_valid() const;
 
-		CanvasShaderData() {}
+		CanvasShaderData();
 		virtual ~CanvasShaderData();
 	};
 
+	struct {
+		// Data must be guaranteed to be erased before the rest on the destructor.
+		CanvasShaderData *default_version_data = nullptr;
+		CanvasShaderRD canvas_shader;
+		RID default_version_rd_shader;
+		RID quad_index_buffer;
+		RID quad_index_array;
+		ShaderCompiler compiler;
+		uint32_t pipeline_compilations[RS::PIPELINE_SOURCE_MAX] = {};
+		Mutex mutex;
+	} shader;
+
 	RendererRD::MaterialStorage::ShaderData *_create_shader_func();
 	static RendererRD::MaterialStorage::ShaderData *_create_shader_funcs() {
 		return static_cast<RendererCanvasRenderRD *>(singleton)->_create_shader_func();
@@ -264,12 +266,23 @@ class RendererCanvasRenderRD : public RendererCanvasRender {
 
 	RID_Owner<CanvasLight> canvas_light_owner;
 
+	struct PositionalShadowRenderPushConstant {
+		float modelview[8];
+		float rotation[4];
+		float direction[2];
+		float z_far;
+		uint32_t pad;
+		float z_near;
+		uint32_t cull_mode;
+		float pad2[2];
+	};
+
 	struct ShadowRenderPushConstant {
 		float projection[16];
 		float modelview[8];
 		float direction[2];
 		float z_far;
-		float pad;
+		uint32_t cull_mode;
 	};
 
 	struct OccluderPolygon {
@@ -309,7 +322,8 @@ class RendererCanvasRenderRD : public RendererCanvasRender {
 	RID_Owner<OccluderPolygon> occluder_polygon_owner;
 
 	enum ShadowRenderMode {
-		SHADOW_RENDER_MODE_SHADOW,
+		SHADOW_RENDER_MODE_DIRECTIONAL_SHADOW,
+		SHADOW_RENDER_MODE_POSITIONAL_SHADOW,
 		SHADOW_RENDER_MODE_SDF,
 	};
 
@@ -321,7 +335,7 @@ class RendererCanvasRenderRD : public RendererCanvasRender {
 	struct {
 		CanvasOcclusionShaderRD shader;
 		RID shader_version;
-		RID render_pipelines[3];
+		RID render_pipelines[2];
 		RID sdf_render_pipelines[2];
 		RD::VertexFormatID vertex_format;
 		RD::VertexFormatID sdf_vertex_format;
@@ -335,51 +349,10 @@ class RendererCanvasRenderRD : public RendererCanvasRender {
 
 	//state that does not vary across rendering all items
 
-	struct State {
-		//state buffer
-		struct Buffer {
-			float canvas_transform[16];
-			float screen_transform[16];
-			float canvas_normal_transform[16];
-			float canvas_modulate[4];
-
-			float screen_pixel_size[2];
-			float time;
-			uint32_t use_pixel_snap;
-
-			float sdf_to_tex[4];
-			float sdf_to_screen[2];
-			float screen_to_sdf[2];
-
-			uint32_t directional_light_count;
-			float tex_to_sdf;
-			uint32_t pad1;
-			uint32_t pad2;
-		};
-
-		LightUniform *light_uniforms = nullptr;
-
-		RID lights_uniform_buffer;
-		RID canvas_state_buffer;
-		RID shadow_sampler;
-		RID shadow_texture;
-		RID shadow_depth_texture;
-		RID shadow_fb;
-		int shadow_texture_size = 2048;
-
-		RID default_transforms_uniform_set;
-
-		uint32_t max_lights_per_render;
-		uint32_t max_lights_per_item;
-
-		double time;
-
-	} state;
-
-	struct PushConstant {
+	struct InstanceData {
 		float world[6];
 		uint32_t flags;
-		uint32_t specular_shininess;
+		uint32_t instance_uniforms_ofs;
 		union {
 			//rect
 			struct {
@@ -403,8 +376,223 @@ class RendererCanvasRenderRD : public RendererCanvasRender {
 		uint32_t lights[4];
 	};
 
+	struct PushConstant {
+		uint32_t base_instance_index;
+		ShaderSpecialization shader_specialization;
+		uint32_t specular_shininess;
+		uint32_t batch_flags;
+	};
+
+	// TextureState is used to determine when a new batch is required due to a change of texture state.
+	struct TextureState {
+		static const uint32_t FILTER_SHIFT = 0;
+		static const uint32_t FILTER_BITS = 3;
+		static const uint32_t FILTER_MASK = (1 << FILTER_BITS) - 1;
+		static const uint32_t REPEAT_SHIFT = FILTER_BITS;
+		static const uint32_t REPEAT_BITS = 2;
+		static const uint32_t REPEAT_MASK = (1 << REPEAT_BITS) - 1;
+		static const uint32_t TEXTURE_IS_DATA_SHIFT = REPEAT_SHIFT + REPEAT_BITS;
+		static const uint32_t TEXTURE_IS_DATA_BITS = 1;
+		static const uint32_t TEXTURE_IS_DATA_MASK = (1 << TEXTURE_IS_DATA_BITS) - 1;
+		static const uint32_t LINEAR_COLORS_SHIFT = TEXTURE_IS_DATA_SHIFT + TEXTURE_IS_DATA_BITS;
+		static const uint32_t LINEAR_COLORS_BITS = 1;
+		static const uint32_t LINEAR_COLORS_MASK = (1 << LINEAR_COLORS_BITS) - 1;
+
+		RID texture;
+		uint32_t other = 0;
+
+		TextureState() {}
+
+		TextureState(RID p_texture, RS::CanvasItemTextureFilter p_base_filter, RS::CanvasItemTextureRepeat p_base_repeat, bool p_texture_is_data, bool p_use_linear_colors) {
+			texture = p_texture;
+			other = (((uint32_t)p_base_filter & FILTER_MASK) << FILTER_SHIFT) |
+					(((uint32_t)p_base_repeat & REPEAT_MASK) << REPEAT_SHIFT) |
+					(((uint32_t)p_texture_is_data & TEXTURE_IS_DATA_MASK) << TEXTURE_IS_DATA_SHIFT) |
+					(((uint32_t)p_use_linear_colors & LINEAR_COLORS_MASK) << LINEAR_COLORS_SHIFT);
+		}
+
+		_ALWAYS_INLINE_ RS::CanvasItemTextureFilter texture_filter() const {
+			return (RS::CanvasItemTextureFilter)((other >> FILTER_SHIFT) & FILTER_MASK);
+		}
+
+		_ALWAYS_INLINE_ RS::CanvasItemTextureRepeat texture_repeat() const {
+			return (RS::CanvasItemTextureRepeat)((other >> REPEAT_SHIFT) & REPEAT_MASK);
+		}
+
+		_ALWAYS_INLINE_ bool linear_colors() const {
+			return (other >> LINEAR_COLORS_SHIFT) & LINEAR_COLORS_MASK;
+		}
+
+		_ALWAYS_INLINE_ bool texture_is_data() const {
+			return (other >> TEXTURE_IS_DATA_SHIFT) & TEXTURE_IS_DATA_MASK;
+		}
+
+		_ALWAYS_INLINE_ bool operator==(const TextureState &p_val) const {
+			return (texture == p_val.texture) && (other == p_val.other);
+		}
+
+		_ALWAYS_INLINE_ bool operator!=(const TextureState &p_val) const {
+			return (texture != p_val.texture) || (other != p_val.other);
+		}
+
+		_ALWAYS_INLINE_ bool is_valid() const { return texture.is_valid(); }
+		_ALWAYS_INLINE_ bool is_null() const { return texture.is_null(); }
+
+		uint32_t hash() const {
+			uint32_t hash = hash_murmur3_one_64(texture.get_id());
+			return hash_murmur3_one_32(other, hash);
+		}
+	};
+
+	struct TextureInfo {
+		TextureState state;
+		RID diffuse;
+		RID normal;
+		RID specular;
+		RID sampler;
+		Vector2 texpixel_size;
+		uint32_t specular_shininess = 0;
+		uint32_t flags = 0;
+	};
+
+	/// A key used to uniquely identify a distinct BATCH_UNIFORM_SET
+	struct RIDSetKey {
+		TextureState state;
+		RID instance_data;
+
+		RIDSetKey() {
+		}
+
+		RIDSetKey(TextureState p_state, RID p_instance_data) :
+				state(p_state),
+				instance_data(p_instance_data) {
+		}
+
+		_ALWAYS_INLINE_ bool operator==(const RIDSetKey &p_val) const {
+			return state == p_val.state && instance_data == p_val.instance_data;
+		}
+
+		_ALWAYS_INLINE_ bool operator!=(const RIDSetKey &p_val) const {
+			return !(*this == p_val);
+		}
+
+		_ALWAYS_INLINE_ uint32_t hash() const {
+			uint32_t h = state.hash();
+			h = hash_murmur3_one_64(instance_data.get_id(), h);
+			return hash_fmix32(h);
+		}
+	};
+
+	static void _before_evict(RendererCanvasRenderRD::RIDSetKey &p_key, RID &p_rid);
+	static void _uniform_set_invalidation_callback(void *p_userdata);
+	static void _canvas_texture_invalidation_callback(bool p_deleted, void *p_userdata);
+
+	typedef LRUCache<RIDSetKey, RID, HashableHasher<RIDSetKey>, HashMapComparatorDefault<RIDSetKey>, _before_evict> RIDCache;
+	RIDCache rid_set_to_uniform_set;
+	/// Maps a CanvasTexture to its associated uniform sets, which must
+	/// be invalidated when the CanvasTexture is updated, such as changing the
+	/// diffuse texture.
+	HashMap<RID, TightLocalVector<RID>> canvas_texture_to_uniform_set;
+
+	struct Batch {
+		// Position in the UBO measured in bytes
+		uint32_t start = 0;
+		uint32_t instance_count = 0;
+		uint32_t instance_buffer_index = 0;
+
+		TextureInfo *tex_info;
+
+		Color modulate = Color(1.0, 1.0, 1.0, 1.0);
+
+		Item *clip = nullptr;
+
+		RID material;
+		CanvasMaterialData *material_data = nullptr;
+
+		const Item::Command *command = nullptr;
+		Item::Command::Type command_type = Item::Command::TYPE_ANIMATION_SLICE; // Can default to any type that doesn't form a batch.
+		ShaderVariant shader_variant = SHADER_VARIANT_QUAD;
+		RD::RenderPrimitive render_primitive = RD::RENDER_PRIMITIVE_TRIANGLES;
+		bool use_lighting = false;
+
+		// batch-specific data
+		union {
+			// TYPE_PRIMITIVE
+			uint32_t primitive_points = 0;
+			// TYPE_PARTICLES
+			uint32_t mesh_instance_count;
+		};
+		bool has_blend = false;
+		uint32_t flags = 0;
+	};
+
+	HashMap<TextureState, TextureInfo, HashableHasher<TextureState>, HashMapComparatorDefault<TextureState>, PagedAllocator<HashMapElement<TextureState, TextureInfo>>> texture_info_map;
+
+	// per-frame buffers
+	struct DataBuffer {
+		LocalVector<RID> instance_buffers;
+	};
+
+	struct State {
+		//state buffer
+		struct Buffer {
+			float canvas_transform[16];
+			float screen_transform[16];
+			float canvas_normal_transform[16];
+			float canvas_modulate[4];
+
+			float screen_pixel_size[2];
+			float time;
+			uint32_t use_pixel_snap;
+
+			float sdf_to_tex[4];
+			float sdf_to_screen[2];
+			float screen_to_sdf[2];
+
+			uint32_t directional_light_count;
+			float tex_to_sdf;
+			float shadow_pixel_size;
+			uint32_t flags;
+		};
+
+		DataBuffer canvas_instance_data_buffers[BATCH_DATA_BUFFER_COUNT];
+		LocalVector<Batch> canvas_instance_batches;
+		uint32_t current_data_buffer_index = 0;
+		uint32_t current_instance_buffer_index = 0;
+		uint32_t current_batch_index = 0;
+		uint32_t last_instance_index = 0;
+		InstanceData *instance_data_array = nullptr;
+
+		uint32_t max_instances_per_buffer = 16384;
+		uint32_t max_instance_buffer_size = 16384 * sizeof(InstanceData);
+
+		Vector<RD::Uniform> batch_texture_uniforms;
+		RID current_batch_uniform_set;
+
+		LightUniform *light_uniforms = nullptr;
+
+		RID lights_storage_buffer;
+		RID canvas_state_buffer;
+		RID shadow_sampler;
+		RID shadow_texture;
+		RID shadow_depth_texture;
+		RID shadow_fb;
+		int shadow_texture_size = 2048;
+
+		RID shadow_occluder_buffer;
+		uint32_t shadow_occluder_buffer_size;
+		RID shadow_ocluder_uniform_set;
+
+		RID default_transforms_uniform_set;
+
+		double time;
+
+	} state;
+
 	Item *items[MAX_RENDER_ITEMS];
 
+	TextureInfo default_texture_info;
+
 	bool using_directional_lights = false;
 	RID default_canvas_texture;
 
@@ -422,9 +610,22 @@ class RendererCanvasRenderRD : public RendererCanvasRender {
 	Color debug_redraw_color;
 	double debug_redraw_time = 1.0;
 
-	inline void _bind_canvas_texture(RD::DrawListID p_draw_list, RID p_texture, RS::CanvasItemTextureFilter p_base_filter, RS::CanvasItemTextureRepeat p_base_repeat, RID &r_last_texture, PushConstant &push_constant, Size2 &r_texpixel_size, bool p_texture_is_data = false); //recursive, so regular inline used instead.
-	void _render_item(RenderingDevice::DrawListID p_draw_list, RID p_render_target, const Item *p_item, RenderingDevice::FramebufferFormatID p_framebuffer_format, const Transform2D &p_canvas_transform_inverse, Item *&current_clip, Light *p_lights, PipelineVariants *p_pipeline_variants, bool &r_sdf_used, const Point2 &p_offset, RenderingMethod::RenderInfo *r_render_info = nullptr);
-	void _render_items(RID p_to_render_target, int p_item_count, const Transform2D &p_canvas_transform_inverse, Light *p_lights, bool &r_sdf_used, bool p_to_backbuffer = false, RenderingMethod::RenderInfo *r_render_info = nullptr);
+	// A structure to store cached render target information
+	struct RenderTarget {
+		// Current render target for the canvas.
+		RID render_target;
+		bool use_linear_colors = false;
+	};
+
+	inline RID _get_pipeline_specialization_or_ubershader(CanvasShaderData *p_shader_data, PipelineKey &r_pipeline_key, PushConstant &r_push_constant, RID p_mesh_instance = RID(), void *p_surface = nullptr, uint32_t p_surface_index = 0, RID *r_vertex_array = nullptr);
+	void _render_batch_items(RenderTarget p_to_render_target, int p_item_count, const Transform2D &p_canvas_transform_inverse, Light *p_lights, bool &r_sdf_used, bool p_to_backbuffer = false, RenderingMethod::RenderInfo *r_render_info = nullptr);
+	void _record_item_commands(const Item *p_item, RenderTarget p_render_target, const Transform2D &p_base_transform, Item *&r_current_clip, Light *p_lights, uint32_t &r_index, bool &r_batch_broken, bool &r_sdf_used, Batch *&r_current_batch);
+	void _render_batch(RD::DrawListID p_draw_list, CanvasShaderData *p_shader_data, RenderingDevice::FramebufferFormatID p_framebuffer_format, Light *p_lights, Batch const *p_batch, RenderingMethod::RenderInfo *r_render_info = nullptr);
+	void _prepare_batch_texture_info(RID p_texture, TextureState &p_state, TextureInfo *p_info);
+	InstanceData *new_instance_data(float *p_world, uint32_t *p_lights, uint32_t p_base_flags, uint32_t p_index, uint32_t p_uniforms_ofs, TextureInfo *p_info);
+	[[nodiscard]] Batch *_new_batch(bool &r_batch_broken);
+	void _add_to_batch(uint32_t &r_index, bool &r_batch_broken, Batch *&r_current_batch);
+	void _allocate_instance_buffer();
 
 	_FORCE_INLINE_ void _update_transform_2d_to_mat2x4(const Transform2D &p_transform, float *p_mat2x4);
 	_FORCE_INLINE_ void _update_transform_2d_to_mat2x3(const Transform2D &p_transform, float *p_mat2x3);
@@ -433,15 +634,16 @@ class RendererCanvasRenderRD : public RendererCanvasRender {
 	_FORCE_INLINE_ void _update_transform_to_mat4(const Transform3D &p_transform, float *p_mat4);
 
 	void _update_shadow_atlas();
+	void _update_occluder_buffer(uint32_t p_size);
 
 public:
-	PolygonID request_polygon(const Vector<int> &p_indices, const Vector<Point2> &p_points, const Vector<Color> &p_colors, const Vector<Point2> &p_uvs = Vector<Point2>(), const Vector<int> &p_bones = Vector<int>(), const Vector<float> &p_weights = Vector<float>()) override;
+	PolygonID request_polygon(const Vector<int> &p_indices, const Vector<Point2> &p_points, const Vector<Color> &p_colors, const Vector<Point2> &p_uvs = Vector<Point2>(), const Vector<int> &p_bones = Vector<int>(), const Vector<float> &p_weights = Vector<float>(), int p_count = -1) override;
 	void free_polygon(PolygonID p_polygon) override;
 
 	RID light_create() override;
 	void light_set_texture(RID p_rid, RID p_texture) override;
 	void light_set_use_shadow(RID p_rid, bool p_enable) override;
-	void light_update_shadow(RID p_rid, int p_shadow_index, const Transform2D &p_light_xform, int p_light_mask, float p_near, float p_far, LightOccluderInstance *p_occluders) override;
+	void light_update_shadow(RID p_rid, int p_shadow_index, const Transform2D &p_light_xform, int p_light_mask, float p_near, float p_far, LightOccluderInstance *p_occluders, const Rect2 &p_light_rect) override;
 	void light_update_directional_shadow(RID p_rid, int p_shadow_index, const Transform2D &p_light_xform, int p_light_mask, float p_cull_distance, const Rect2 &p_clip_rect, LightOccluderInstance *p_occluders) override;
 
 	virtual void render_sdf(RID p_render_target, LightOccluderInstance *p_occluders) override;
@@ -455,6 +657,7 @@ public:
 	virtual void set_shadow_texture_size(int p_size) override;
 
 	void set_debug_redraw(bool p_enabled, double p_time, const Color &p_color) override;
+	uint32_t get_pipeline_compilations(RS::PipelineSource p_source) override;
 
 	void set_time(double p_time);
 	void update() override;
@@ -462,5 +665,3 @@ public:
 	RendererCanvasRenderRD();
 	~RendererCanvasRenderRD();
 };
-
-#endif // RENDERER_CANVAS_RENDER_RD_H
