@@ -28,17 +28,17 @@
 /* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
 /**************************************************************************/
 
-#ifndef TEST_IMAGE_H
-#define TEST_IMAGE_H
+#pragma once
 
 #include "core/io/image.h"
 #include "core/os/os.h"
 
 #include "tests/test_utils.h"
-#include "thirdparty/doctest/doctest.h"
 
 #include "modules/modules_enabled.gen.h"
 
+#include "thirdparty/doctest/doctest.h"
+
 namespace TestImage {
 
 TEST_CASE("[Image] Instantiation") {
@@ -113,7 +113,7 @@ TEST_CASE("[Image] Saving and loading") {
 	// Load BMP
 	Ref<Image> image_bmp = memnew(Image());
 	Ref<FileAccess> f_bmp = FileAccess::open(TestUtils::get_data_path("images/icon.bmp"), FileAccess::READ, &err);
-	REQUIRE(!f_bmp.is_null());
+	REQUIRE(f_bmp.is_valid());
 	PackedByteArray data_bmp;
 	data_bmp.resize(f_bmp->get_length() + 1);
 	f_bmp->get_buffer(data_bmp.ptrw(), f_bmp->get_length());
@@ -126,20 +126,48 @@ TEST_CASE("[Image] Saving and loading") {
 	// Load JPG
 	Ref<Image> image_jpg = memnew(Image());
 	Ref<FileAccess> f_jpg = FileAccess::open(TestUtils::get_data_path("images/icon.jpg"), FileAccess::READ, &err);
-	REQUIRE(!f_jpg.is_null());
+	REQUIRE(f_jpg.is_valid());
 	PackedByteArray data_jpg;
 	data_jpg.resize(f_jpg->get_length() + 1);
 	f_jpg->get_buffer(data_jpg.ptrw(), f_jpg->get_length());
 	CHECK_MESSAGE(
 			image_jpg->load_jpg_from_buffer(data_jpg) == OK,
 			"The JPG image should load successfully.");
+
+	Ref<Image> image_grayscale_jpg = memnew(Image());
+	Ref<FileAccess> f_grayscale_jpg = FileAccess::open(TestUtils::get_data_path("images/grayscale.jpg"), FileAccess::READ, &err);
+	REQUIRE(f_grayscale_jpg.is_valid());
+	PackedByteArray data_grayscale_jpg;
+	data_grayscale_jpg.resize(f_grayscale_jpg->get_length() + 1);
+	f_grayscale_jpg->get_buffer(data_grayscale_jpg.ptrw(), f_grayscale_jpg->get_length());
+	CHECK_MESSAGE(
+			image_jpg->load_jpg_from_buffer(data_grayscale_jpg) == OK,
+			"The grayscale JPG image should load successfully.");
+
+	// Save JPG
+	const String save_path_jpg = TestUtils::get_temp_path("image.jpg");
+	CHECK_MESSAGE(image->save_jpg(save_path_jpg) == OK,
+			"The image should be saved successfully as a .jpg file.");
+
+#ifdef MODULE_SVG_ENABLED
+	// Load SVG with embedded jpg image
+	Ref<Image> image_svg = memnew(Image());
+	Ref<FileAccess> f_svg = FileAccess::open(TestUtils::get_data_path("images/embedded_jpg.svg"), FileAccess::READ, &err);
+	REQUIRE(f_svg.is_valid());
+	PackedByteArray data_svg;
+	data_svg.resize(f_svg->get_length() + 1);
+	f_svg->get_buffer(data_svg.ptrw(), f_svg->get_length());
+	CHECK_MESSAGE(
+			image_svg->load_svg_from_buffer(data_svg) == OK,
+			"The SVG image should load successfully.");
+#endif // MODULE_SVG_ENABLED
 #endif // MODULE_JPG_ENABLED
 
 #ifdef MODULE_WEBP_ENABLED
 	// Load WebP
 	Ref<Image> image_webp = memnew(Image());
 	Ref<FileAccess> f_webp = FileAccess::open(TestUtils::get_data_path("images/icon.webp"), FileAccess::READ, &err);
-	REQUIRE(!f_webp.is_null());
+	REQUIRE(f_webp.is_valid());
 	PackedByteArray data_webp;
 	data_webp.resize(f_webp->get_length() + 1);
 	f_webp->get_buffer(data_webp.ptrw(), f_webp->get_length());
@@ -151,7 +179,7 @@ TEST_CASE("[Image] Saving and loading") {
 	// Load PNG
 	Ref<Image> image_png = memnew(Image());
 	Ref<FileAccess> f_png = FileAccess::open(TestUtils::get_data_path("images/icon.png"), FileAccess::READ, &err);
-	REQUIRE(!f_png.is_null());
+	REQUIRE(f_png.is_valid());
 	PackedByteArray data_png;
 	data_png.resize(f_png->get_length() + 1);
 	f_png->get_buffer(data_png.ptrw(), f_png->get_length());
@@ -163,7 +191,7 @@ TEST_CASE("[Image] Saving and loading") {
 	// Load TGA
 	Ref<Image> image_tga = memnew(Image());
 	Ref<FileAccess> f_tga = FileAccess::open(TestUtils::get_data_path("images/icon.tga"), FileAccess::READ, &err);
-	REQUIRE(!f_tga.is_null());
+	REQUIRE(f_tga.is_valid());
 	PackedByteArray data_tga;
 	data_tga.resize(f_tga->get_length() + 1);
 	f_tga->get_buffer(data_tga.ptrw(), f_tga->get_length());
@@ -443,5 +471,3 @@ TEST_CASE("[Image] Convert image") {
 }
 
 } // namespace TestImage
-
-#endif // TEST_IMAGE_H
